------------------------------------------------------------------------------
--  Module name:    COMPRESS.E (needs Euphoria 1.5+)                        --
--  Description:    Compress and decompress files.                          --
--  Version:        1.0                                                     --
--  Date:           April 7,1997                                            --
--  Author:         Daniel Berstein                                         --
--  Email:          architek@geocities.com                                  -- 
--  Webpage:        http://www.geocities.com/SiliconValley/Heights/9316     --
--  Changes:        (none)                                                  --
------------------------------------------------------------------------------
-- Documentation:                                                           --
--                                                                          --
--      This code is a translation to Euphoria from a C source made by      --
--  Nico E. Vries, which derives from the algorithm suggested by Jussi      --
--  Puttonen, Tino Raita and Jukka Teuhola.                                 --
--      The original code was released to the public domain, so I assume I  --
--  can use it. I make THIS (read "my") code also available to the public   --
--  domain. I hope you find this stuff useful. I only ask you to mention me --
--  if you use it in a "profesional" program.                               --
--      Just to make sure you understand my mood: I'm not responsable for   --
--  *ANY* damage this code could produce.                                   --
--                                                                          --
--  The module consist of two functions:                                   --
--                                                                          --
--        compress( <InFile> , <OutFile> )                                  --
--      decompress( <InFile> , <OutFile> )                                  --
--                                                                          --
--  InFile and Outfile are required parameters.                             --
--  Return values:  0 = FAIL (could not open/create files)                  --
--                  1 = SUCCESS (all done!)                                 --
------------------------------------------------------------------------------

function INDEX(integer p1, integer p2)
    integer index
    index = xor_bits((p1*128),p2) + 1
    return index
end function            --End INDEX

--COMPRESS
global function compress(sequence infile, sequence outfile)
    --Declaring variables
    sequence pcTable
    integer p1,p2,c,i,ctr,bctr,mask
    sequence buf
    object compIn, compOut

    --Open files, if unable to do it, return error (0)
    compIn = open(infile, "rb")
    if compIn = -1 then
	return 0
    end if
    compOut = open(outfile, "wb")
    if compOut = -1 then
	return 0
    end if

    --Initialize variables
    pcTable = repeat(32, 32768)
    buf = repeat(0,8)
    p1 = 0
    p2 = 0
    bctr = 0
    ctr = 0
    mask = 0

    --Main loop
    c = getc(compIn)
    while c != -1 do
	if pcTable[INDEX(p1,p2)] = c then
	    mask = xor_bits(mask,power(2,ctr))
	else
	    pcTable[INDEX(p1,p2)] = c
	    buf[bctr+1] = c
	    bctr = bctr + 1
	end if
	ctr = ctr + 1
	if ctr = 8 then
	    puts(compOut, mask)
	    i = 0
	    while i < bctr do
		puts(compOut, buf[i+1])
		i = i + 1
	    end while
	    ctr = 0
	    bctr = 0
	    mask = 0
	end if
	p1 = p2
	p2 = c
	c = getc(compIn)
    end while
    
    --Flush characters in buffer
    if ctr then
	puts(compOut, mask)
	i = 0
	while i < bctr do
	    puts(compOut, buf[i+1])
	    i = i + 1
	end while
    end if
    --Compression succesful, close files and return (1)
    close(compIn)
    close(compOut)
    return 1
end function            --End COMPRESS

--Decompress
global function decompress(sequence infile, sequence outfile)
    --Declaring variables
    sequence pcTable
    integer p1,p2,ctr,mask,ci,co
    object decompIn, decompOut
    integer test

    --Open files, if unable to do it, return error (0)
    decompIn = open(infile, "rb")
    if decompIn = -1 then
	return 0
    end if
    decompOut = open(outfile, "wb")
    if decompOut = -1 then
	return 0
    end if

    --Initialize variables
    p1 = 0
    p2 = 0
    mask = 0
    pcTable = repeat(32,32768)
    ci = getc(decompIn)

    --Main loop
    while ci != -1 do
	mask = ci
	ctr = 0
	while ctr < 8 do
	    test = and_bits(mask,power(2,ctr))
	    if test then
		co = pcTable[INDEX(p1,p2)]
	    else
		co = getc(decompIn)
		if co = -1 then
		    --Decompression completed, close files and return (1)
		    close(decompIn)
		    close(decompOut)
		    return 1
		else
		    pcTable[INDEX(p1,p2)] = co
		end if
	    end if
	    puts(decompOut, co)
	    p1 = p2
	    p2 = co
	    ctr = ctr + 1
	end while
	ci = getc(decompIn)
    end while
    --Decompression completed, close files and return (1)
    close(decompIn)
    close(decompOut)
    return 1
end function            --End DECOMPRESS
