-- Test program for COMPRESS.E module
-- File compressor and decompressor.
-- Author: Daniel Berstein

include compress.e
include file.e
include wildcard.e

integer done
sequence in, out, mode
integer error

procedure start()
    object aux, aux2
    -- Read the command line
    aux = command_line()
    -- Only two parameters allowed (input and output files)
    if length(aux) != 5 then
	error = 1
	return
    end if
    -- Assign mode and filenames
    mode = aux[3]
    in = aux[4]
    out = aux[5]
    -- Check mode
    mode = upper(mode)
    if mode[1] !='D' and mode[1] !='C' then
	error = 1
	return
    end if
    -- Don't mess with directories
    if not compare(in,".") or not compare(in,"..") or not compare(out,".") or
      not compare(out,"..") then
	error = 1
	return
    end if
    aux = dir(in)
    aux2 = dir(out)
    if atom(aux) then
	error = 1
	return
    elsif find('d', aux[1][D_ATTRIBUTES]) then
	error = 1
	return
    elsif sequence(aux2) then
	if find('d', aux2[1][D_ATTRIBUTES]) then
	    error = 1
	    return
	end if
    end if
    return
end procedure

function size(sequence filename)
    sequence aux
    aux = dir(filename)
    return aux[1][D_SIZE]
end function

procedure comp()
    -- Compressing file
    puts(1, "Compressing file: " & in & " into " & out & "\n")
    done = compress(in,out)
    -- Check for errors
    if not done then
	puts(1,"Couldn't compress, error opening/creating files\n")
	abort(0)
    end if
    -- All OK, give stats
    puts(1, "Original size: ")
    print(1, size(in))
    puts(1, "\nCompressed size: ")
    print(1, size(out))
    puts(1, "\n")
    return
end procedure

procedure decomp()
    -- Decompressing file
    puts(1, "Decompressing file: " & in & " into " & out & "\n")
    done = decompress(in,out)
    -- Check for errors
    if not done then
	puts(1,"Couldn't decompress, error opening/creating files\n")
	abort(0)
    end if
    -- All OK, give stats
    puts(1, "Compressed size: ")
    print(1, size(in))
    puts(1, "\nExpanded size: ")
    print(1, size(out))
    puts(1, "\n")
    return
end procedure

error = 0
start()
if error then
    puts(1,"\nEuphoria File Compressor - Daniel Berstein, april 1997\n")
    puts(1,"Parameters:   [c/d] <InFile> <OutFile>\n")
    puts(1,"               c = compress InFile into OutFile\n")
    puts(1,"               d = decompress InFile into OutFile\n\n")
    abort(0)
end if
clear_screen()
if mode[1] = 'C' then
    comp()
else
    decomp()
end if
